import logging
import os
import time
from urllib.parse import quote

import pyotp
import requests
import undetected_chromedriver as uc
from selenium.webdriver.common.by import By

from dvarpal.config import SessionConfig


class SessionManager:

    def __init__(self, config: SessionConfig = SessionConfig()):
        """
        If config object is None, loads config from `${HOME_DIR}/dvarpal.yaml.
        :param config:
        """
        super().__init__()
        self.__logger = logging.getLogger(self.__class__.__name__)
        self.__access_token = ''
        self.__config = config

    def generate_access_token(self):

        # check if session_token file exists. If it exists, load token from file.
        self._load_access_token_from_file()

        if self.__access_token is not None and self.is_session_valid():
            self.__logger.info("Access token is still valid. Nothing to do.")
            return

        access_token = self._get_access_token()
        self.__access_token = access_token
        self.__logger.info(f'New access token acquired: {access_token}' )
        self._save_access_token(access_token)

    def get_access_token(self):
        return self.__access_token

    def is_session_valid(self) -> bool:
        url = self.__config.session_validation_url

        headers = {
            "Api-Version": "2.0",
            "Authorization": f"Bearer {self.__access_token}",
            "Accept": "application/json"
        }

        try:
            response = requests.get(url, headers=headers)
            body = response.text
            self.__logger.info(f"Ping response: http-status: {response.status_code} > body: {body}")
            if response.status_code == 200:
                return True
        except requests.exceptions.RequestException as e:
            self.__logger.error("Ping failed!", e)

        return False

    def _save_access_token(self, access_token: str):
        try:
            with open(self.__config.access_token_file, 'w') as file:
                file.write(access_token)
            self.__logger.info(f"Access token successfully written to {self.__config.access_token_file}")
        except Exception as e:
            raise Exception(f"Failed to save access token to file {self.__config.access_token_file}", e)

    def _load_access_token_from_file(self):
        if not os.path.exists(self.__config.access_token_file):
            return

        with open(self.__config.access_token_file, 'r') as file:
            content = file.read()
        self.__access_token = content
        self.__logger.info(f'Loaded access token from file {self.__config.access_token_file}')

    def _get_access_code(self) -> str:

        options = uc.ChromeOptions()
        if self.__config.browser_headless:
            options.add_argument("--headless") # Do not enable this. Fails detecting OTP input
        options.add_argument(f"--user-agent={self.__config.browser_useragent}")
        driver = uc.Chrome(options=options)
        try:

            authn_url = (f'{self.__config.authn_url}?'
                         f'response_type=code&'
                         f'client_id={self.__config.client_id}&'
                         f'redirect_uri={quote(self.__config.redirect_uri)}')
            driver.get(authn_url)
            time.sleep(5)
            self.__logger.info('==> browser launched')

            phone_no = driver.find_element(By.ID, "mobileNum")
            phone_no.send_keys(self.__config.mobile)
            self.__logger.info('==> mobile number entered')

            get_otp_btn = driver.find_element(By.ID, "getOtp")
            get_otp_btn.submit()
            self.__logger.info('==> OTP requested')
            time.sleep(5)

            otp = driver.find_element(By.ID, "otpNum")
            totp = pyotp.TOTP(self.__config.totp_secret_key).now()
            otp.send_keys(totp)
            self.__logger.info('==> TOTP entered')
            continue_btn = driver.find_element(By.ID, "continueBtn")
            continue_btn.submit()
            self.__logger.info('==> TOTP submitted')
            time.sleep(5)

            pin = driver.find_element(By.ID, "pinCode")
            pin.send_keys(self.__config.pin)
            self.__logger.info('==> PIN entered')
            submit = driver.find_element(By.ID, "pinContinueBtn")
            submit.click()
            self.__logger.info('==> PIN submitted')
            time.sleep(5)

            url = driver.current_url
            initial_access_code = url.split('code=')[1]
            access_code = initial_access_code.split('&')[0]
            self.__logger.debug(f'access_code: {access_code}')
        finally:
            if driver is not None:
                driver.close()
                driver.quit()

        return access_code

    def _get_access_token(self) -> str:
        s = requests.Session()
        access_code = self._get_access_code()
        # header data that needs to be sent to the post method
        headers = {"accept": "application/json", "Api-Version": "2.0",
                   "content-type": "application/x-www-form-urlencoded"}
        # the main data that needs to be sent to the post method
        data = {'code': access_code,
                'client_id': self.__config.client_id,
                'client_secret': self.__config.client_secret,
                'redirect_uri': self.__config.redirect_uri,
                'grant_type': 'authorization_code'
                }
        # post method consists of three parameters url, header and data
        resp = s.post(url=self.__config.authz_url, headers=headers, data=data)
        # test for the status code else throws error
        assert resp.status_code == 200, f"Error in r3:\n {resp.json()}"

        json_response = resp.json()
        # read the access token from the json response
        access_token = json_response['access_token']
        self.__logger.debug(f'access_token: {access_token}')
        return access_token



import logging
import os

from dvarpal.config import SessionConfig
from dvarpal.session import SessionManager




import logging
import os
import time
from urllib.parse import quote

import pyotp
import requests
import undetected_chromedriver as uc
from selenium.webdriver.common.by import By

class TestSessionManager:
    def __init__(self):
        self.logger = logging.getLogger(self.__class__.__name__)

    def test_session_generation_no_file(self):
        session_config = SessionConfig()
        print(session_config.__dict__)
        self.__delete_file_safely(session_config.access_token_file)

        session_manager = SessionManager(session_config)
        session_manager.generate_access_token()

    def test_session_generation_empty_file(self):
        session_config = SessionConfig()
        self.__create_access_token_file(session_config.access_token_file, '')

        session_manager = SessionManager(session_config)
        session_manager.generate_access_token()

    def test_session_generation_invalid_token_in_file(self):
        session_config = SessionConfig()
        self.__create_access_token_file(session_config.access_token_file, 'CRAP')

        session_manager = SessionManager(session_config)
        session_manager.generate_access_token()

    def test_session_generation_valid_token_in_file(self):
        session_manager = SessionManager(SessionConfig())
        session_manager.generate_access_token()

    def __delete_file_safely(self, file_path):
        try:
            os.remove(file_path)
            self.logger.info(f"File {file_path} deleted successfully.")
        except FileNotFoundError:
            self.logger.info(f"File {file_path} does not exist.")
        except Exception as e:
            self.logger.info(f"An error occurred while deleting the file: {e}")

    def __create_access_token_file(self, file_path, content):
        try:
            with open(file_path, 'w') as file:
                file.write(content)
            self.logger.info(f"File created at: {file_path} with content '{content}'.")
        except Exception as e:
            self.logger.info(f"An error occurred while creating the file: {e}")

    def foochoda(self):
        options = uc.ChromeOptions()
        # options.add_argument("--headless") # Do not enable this. Fails detecting OTP input
        options.add_argument(f"--headless")
        options.add_argument(f"--user-agent=Mozilla/5.0 (X11; Ubuntu; Linux x86_64; rv:15.0) Gecko/20100101 Firefox/15.0.1")
        driver = uc.Chrome(options=options)
        try:

            authn_url = ('https://practicetestautomation.com/practice-test-login/')
            driver.get(authn_url)

            phone_no = driver.find_element(By.NAME, "username")
            phone_no.send_keys('student')
            get_otp_btn = driver.find_element(By.NAME, "password")
            get_otp_btn.send_keys('Password123')
            submit = driver.find_element(By.ID, "submit")
            submit.click()
            # driver.implicitly_wait(10)
            time.sleep(3)

            otp = driver.find_element(By.XPATH, "/html/body/div/div/section/div/div/article/div[2]/p[1]/strong")
            text = otp.text
            print(f'outcome = {text}')
        finally:
            if driver is not None:
                driver.close()
                driver.quit()

if __name__ == '__main__':
    logging.basicConfig(level=logging.INFO)
    tester = TestSessionManager()
    # tester.foochoda()
    tester.test_session_generation_no_file()
    tester.test_session_generation_empty_file()
    tester.test_session_generation_invalid_token_in_file()




from dvarpal.config import SessionConfig
from dvarpal.session_firefox import SessionManager

import logging
import os


class TestSessionManager:
    def __init__(self):
        self.logger = logging.getLogger(self.__class__.__name__)

    def test_session_generation_no_file(self):
        session_config = SessionConfig()
        print(session_config.__dict__)
        self.__delete_file_safely(session_config.access_token_file)

        session_manager = SessionManager(session_config)
        session_manager.generate_access_token()

    def test_session_generation_empty_file(self):
        session_config = SessionConfig()
        self.__create_access_token_file(session_config.access_token_file, '')

        session_manager = SessionManager(session_config)
        session_manager.generate_access_token()

    def test_session_generation_invalid_token_in_file(self):
        session_config = SessionConfig()
        self.__create_access_token_file(session_config.access_token_file, 'CRAP')

        session_manager = SessionManager(session_config)
        session_manager.generate_access_token()

    def test_session_generation_valid_token_in_file(self):
        session_manager = SessionManager(SessionConfig())
        session_manager.generate_access_token()

    def __delete_file_safely(self, file_path):
        try:
            os.remove(file_path)
            self.logger.info(f"File {file_path} deleted successfully.")
        except FileNotFoundError:
            self.logger.info(f"File {file_path} does not exist.")
        except Exception as e:
            self.logger.info(f"An error occurred while deleting the file: {e}")

    def __create_access_token_file(self, file_path, content):
        try:
            with open(file_path, 'w') as file:
                file.write(content)
            self.logger.info(f"File created at: {file_path} with content '{content}'.")
        except Exception as e:
            self.logger.info(f"An error occurred while creating the file: {e}")


if __name__ == '__main__':
    logging.basicConfig(level=logging.INFO)
    tester = TestSessionManager()
    tester.test_session_generation_no_file()
    tester.test_session_generation_empty_file()
    tester.test_session_generation_invalid_token_in_file()




import logging
import os
import time
from urllib.parse import quote

import pyotp
import requests

from selenium import webdriver
from selenium.webdriver.common.by import By
from selenium.webdriver.firefox.options import Options

from dvarpal.config import SessionConfig


class SessionManager:

    def __init__(self, config: SessionConfig = SessionConfig()):
        """
        If config object is None, loads config from `${HOME_DIR}/dvarpal.yaml.
        :param config:
        """
        super().__init__()
        self.__logger = logging.getLogger(self.__class__.__name__)
        self.__access_token = ''
        self.__config = config

    def generate_access_token(self):

        # check if session_token file exists. If it exists, load token from file.
        self._load_access_token_from_file()

        if self.__access_token is not None and self.is_session_valid():
            self.__logger.info("Access token is still valid. Nothing to do.")
            return

        access_token = self._get_access_token()
        self.__access_token = access_token
        self.__logger.info(f'New access token acquired: {access_token}' )
        self._save_access_token(access_token)

    def get_access_token(self):
        return self.__access_token

    def is_session_valid(self) -> bool:
        url = self.__config.session_validation_url

        headers = {
            "Api-Version": "2.0",
            "Authorization": f"Bearer {self.__access_token}",
            "Accept": "application/json"
        }

        try:
            response = requests.get(url, headers=headers)
            body = response.text
            self.__logger.info(f"Ping response: http-status: {response.status_code} > body: {body}")
            if response.status_code == 200:
                return True
        except requests.exceptions.RequestException as e:
            self.__logger.error("Ping failed!", e)

        return False

    def _save_access_token(self, access_token: str):
        try:
            with open(self.__config.access_token_file, 'w') as file:
                file.write(access_token)
            self.__logger.info(f"Access token successfully written to {self.__config.access_token_file}")
        except Exception as e:
            raise Exception(f"Failed to save access token to file {self.__config.access_token_file}", e)

    def _load_access_token_from_file(self):
        if not os.path.exists(self.__config.access_token_file):
            return

        with open(self.__config.access_token_file, 'r') as file:
            content = file.read()
        self.__access_token = content
        self.__logger.info(f'Loaded access token from file {self.__config.access_token_file}')

    def _get_access_code(self) -> str:

        options = Options()
        # Explicitly use Firefox ESR
        options.binary_location = "/usr/bin/firefox-esr"
        if self.__config.browser_headless:
            options.add_argument("--headless")
        options.set_preference("general.useragent.override", self.__config.browser_useragent)
        # options.add_argument(f"--user-agent={self.__config.browser_useragent}")

        driver = webdriver.Firefox(options=options)
        try:

            authn_url = (f'{self.__config.authn_url}?'
                         f'response_type=code&'
                         f'client_id={self.__config.client_id}&'
                         f'redirect_uri={quote(self.__config.redirect_uri)}')
            driver.get(authn_url)
            time.sleep(5)
            self.__logger.info('==> browser launched')

            phone_no = driver.find_element(By.ID, "mobileNum")
            phone_no.send_keys(self.__config.mobile)
            self.__logger.info('==> mobile number entered')

            get_otp_btn = driver.find_element(By.ID, "getOtp")
            get_otp_btn.submit()
            self.__logger.info('==> OTP requested')
            time.sleep(5)

            otp = driver.find_element(By.ID, "otpNum")
            totp = pyotp.TOTP(self.__config.totp_secret_key).now()
            otp.send_keys(totp)
            self.__logger.info('==> TOTP entered')
            continue_btn = driver.find_element(By.ID, "continueBtn")
            continue_btn.submit()
            self.__logger.info('==> TOTP submitted')
            time.sleep(5)

            pin = driver.find_element(By.ID, "pinCode")
            pin.send_keys(self.__config.pin)
            self.__logger.info('==> PIN entered')
            submit = driver.find_element(By.ID, "pinContinueBtn")
            submit.click()
            self.__logger.info('==> PIN submitted')
            time.sleep(5)

            url = driver.current_url
            initial_access_code = url.split('code=')[1]
            access_code = initial_access_code.split('&')[0]
            self.__logger.debug(f'access_code: {access_code}')
        finally:
            if driver is not None:
                driver.close()
                driver.quit()

        return access_code

    def _get_access_token(self) -> str:
        s = requests.Session()
        access_code = self._get_access_code()
        # header data that needs to be sent to the post method
        headers = {"accept": "application/json", "Api-Version": "2.0",
                   "content-type": "application/x-www-form-urlencoded"}
        # the main data that needs to be sent to the post method
        data = {'code': access_code,
                'client_id': self.__config.client_id,
                'client_secret': self.__config.client_secret,
                'redirect_uri': self.__config.redirect_uri,
                'grant_type': 'authorization_code'
                }
        # post method consists of three parameters url, header and data
        resp = s.post(url=self.__config.authz_url, headers=headers, data=data)
        # test for the status code else throws error
        assert resp.status_code == 200, f"Error in r3:\n {resp.json()}"

        json_response = resp.json()
        # read the access token from the json response
        access_token = json_response['access_token']
        self.__logger.debug(f'access_token: {access_token}')
        return access_token



from setuptools import setup, find_packages

setup(
    name='dvarpal',
    version='2.0.0',
    description='Automated broker login and token generator using Firefox ESR and Selenium',
    long_description=open("README.md").read(),
    long_description_content_type="text/markdown",
    author='Amit Kumar Sharma',
    author_email='amit.official@gmail.com',
    url='https://github.com/sharmaak/dvarpal',
    packages=find_packages(),
    include_package_data=True,
    install_requires=[
        'PyYAML~=6.0.2',
        'requests',
        'pyotp',
        'upstox-python-sdk',
        'undetected_chromedriver',
        'selenium',
    ],
    classifiers=[
        "Development Status :: 4 - Beta",
        "Intended Audience :: Developers",
        "License :: OSI Approved :: MIT License",
        "Programming Language :: Python :: 3",
        "Operating System :: POSIX :: Linux",
    ],
    python_requires='>=3.10',
)



import logging
import os

import yaml


class SessionConfig:

    def __init__(self):
        cfg = SessionConfig._load_config_from_file()
        self.access_token_file = cfg.get('access_token_file') if cfg.get('access_token_file') else os.path.join(SessionConfig.get_home_dir(), 'dvarpal_session')

        self.authn_url: str = cfg['authn_url']
        self.authz_url: str = cfg['authz_url']
        self.redirect_uri: str = cfg['redirect_uri']
        self.session_validation_url: str = cfg['session_validation_url']
        self.browser_useragent = cfg.get('browser_useragent') if cfg.get('browser_useragent') is not None else 'Mozilla/5.0 (X11; Ubuntu; Linux x86_64; rv:15.0) Gecko/20100101 Firefox/15.0.1'
        self.browser_headless = False if cfg.get('browser_headless') is None else cfg.get('browser_headless')

        self.client_id: str = cfg['client_id']
        self.client_secret: str = cfg['client_secret']
        self.mobile: str = cfg['mobile']
        self.totp_secret_key: str = cfg['totp_secret_key']
        self.pin: str = cfg['pin']

    @staticmethod
    def _load_config_from_file():
        # Choose home directory env variable based on OS. os.name = 'nt' for windows.
        config_file = os.path.join(SessionConfig.get_home_dir(), 'dvarpal.yaml')
        if not os.path.exists(config_file):
            pass

        with open(config_file, 'r') as f:
            config_data = yaml.safe_load(f)
        # logging.info(f'Loaded dvarpal config from file {config_file}')
        return config_data

    @staticmethod
    def get_home_dir():
        return os.getenv('USERPROFILE') if os.name == 'nt' else os.getenv('HOME')

